@@ -55,21 +55,13 @@
 #include <ibsim.h>
 #include "sim.h"
 
-#define IBSIM_VERSION "0.7"
+#define IBSIM_VERSION "0.12"
 
 #undef DEBUG
 #define PDEBUG	if (parsedebug) IBWARN
 #define DEBUG	if (simverb > 1 || ibdebug) IBWARN
 #define VERB	if (simverb || ibdebug) IBWARN
 
-extern int maxnetnodes;
-extern int maxnetswitches;
-extern int maxnetports;
-extern int maxlinearcap;
-extern int maxmcastcap;
-extern int maxnetaliases;
-extern int ignoreduplicate;
-
 int ibdebug;
 int parsedebug;
 int simverb;
@@ -81,6 +73,7 @@ static FILE *simout;
 static int listen_to_port = IBSIM_DEFAULT_SERVER_PORT;
 static int remote_mode = 0;
 static char* socket_basename;
+static int console = 1;
 
 static size_t make_name(union name_t *name, uint32_t addr, unsigned short port,
 			const char *fmt, ...)
@@ -345,7 +338,9 @@ static void set_issm(Port *port, unsigned issm)
 
 static int sim_ctl_set_issm(Client * cl, struct sim_ctl * ctl)
 {
-	int issm = *(int *)ctl->data;
+	int issm;
+
+	memcpy(&issm, &ctl->data[0], sizeof(int));
 
 	VERB("set issm %d port %" PRIx64, issm, cl->port->portguid);
 	cl->issm = issm;
@@ -517,44 +512,6 @@ static int sim_read_pkt(int fd, int client)
 	return -1;		// never reached
 }
 
-int sim_cmd_file(FILE * f, char *s)
-{
-	char line[4096];
-	FILE *cmd_file;
-	char *p;
-
-	s++;
-	while (isspace(*s))
-		s++;
-
-	if (!s || !*s) {
-		fprintf(f, "do_cmd_from_file: no file name - skip\n");
-		return -1;
-	}
-
-	p = s + strlen(s) - 1;
-	while (isspace(*p)) {
-		*p = '\0';
-		p--;
-	}
-
-	cmd_file = fopen(s, "r");
-	if (!cmd_file) {
-		fprintf(f, "do_cmd_from_file: cannot open file \'%s\': %s\n",
-			s, strerror(errno));
-		return -1;
-	}
-
-	while (fgets(line, sizeof(line) - 1, cmd_file) != NULL) {
-		if((p = strchr(line, '\n')) != NULL)
-			*p = '\0';
-		do_cmd(line, f);
-	}
-
-	fclose(cmd_file);
-	return 0;
-}
-
 static int sim_init_net(char *netconf, FILE * out)
 {
 	DEBUG("reading %s", netconf);
@@ -581,7 +538,8 @@ static int sim_init_console(FILE *out)
 	fprintf(simout, "MaxNetPorts    = %d\n", maxnetports);
 	fprintf(simout, "MaxLinearCap   = %d\n", maxlinearcap);
 	fprintf(simout, "MaxMcastCap    = %d\n", maxmcastcap);
-	fprintf(simout, "sim%s> ", netstarted ? "" : " (inactive)");
+	if (console)
+		fprintf(simout, "sim%s> ", netstarted ? "" : " (inactive)");
 	fflush(simout);
 	return 0;
 }
@@ -623,12 +581,13 @@ static int sim_run(int con_fd)
 	for (;;) {
 		FD_ZERO(&rfds);
 		FD_SET(simctl, &rfds);
-		FD_SET(con_fd, &rfds);
+		if (console)
+			FD_SET(con_fd, &rfds);
 		for (i = 0; i < IBSIM_MAX_CLIENTS; i++)
 			if (clients[i].pid)
 				FD_SET(clients[i].fd, &rfds);
 
-		if (select(maxfd + 1, &rfds, NULL, NULL, 0) < 0)
+		if (select(maxfd + 1, &rfds, NULL, NULL, NULL) < 0)
 			break;	// timeout or error
 
 		if (FD_ISSET(simctl, &rfds))
@@ -702,15 +661,15 @@ Client *find_client(Port * port, int response, int qp, uint64_t trid)
 			return cl;
 	}
 	DEBUG("no client found");
-	return 0;
+	return NULL;
 }
 
-void usage(char *prog_name)
+static void usage(char *prog_name)
 {
 	fprintf(stderr,
 		"Usage: %s [-f outfile -d(ebug) -p(arse_debug) -s(tart) -v(erbose) "
 		"-I(gnore_duplicate) -N nodes -S switches -P ports -L linearcap"
-		" -M mcastcap -r(emote_mode) -l(isten_to_port) <port>] <netfile>\n",
+		" -M mcastcap -r(emote_mode) -l(isten_to_port) <port> -n(o-console)] <netfile>\n",
 		prog_name);
 	fprintf(stderr, "%s %s\n", prog_name, IBSIM_VERSION);
 
@@ -719,30 +678,29 @@ void usage(char *prog_name)
 
 int main(int argc, char **argv)
 {
-	extern int alloc_core(void);
-	extern void free_core(void);
-	char *outfname = 0, *netfile;
+	char *outfname = NULL, *netfile;
 	FILE *infile, *outfile;
 	int status;
 
-	static char const str_opts[] = "rf:dpvIsN:S:P:L:M:l:Vhu";
+	static char const str_opts[] = "rf:dpvIsN:S:P:L:M:l:Vhun";
 	static const struct option long_opts[] = {
-		{"remote", 0, 0, 'r'},
-		{"file", 1, 0, 'f'},
-		{"Nodes", 1, 0, 'N'},
-		{"Switches", 1, 0, 'S'},
-		{"Ports", 1, 0, 'P'},
-		{"Linearcap", 1, 0, 'L'},
-		{"Mcastcap", 1, 0, 'M'},
-	        {"listen", 1, 0, 'l'},
-		{"Ignoredups", 0, 0, 'I'},
-		{"start", 0, 0, 's'},
-		{"debug", 0, 0, 'd'},
-		{"parsedebug", 0, 0, 'p'},
-		{"verbose", 0, 0, 'v'},
-		{"Version", 0, 0, 'V'},
-		{"help", 0, 0, 'h'},
-		{"usage", 0, 0, 'u'},
+		{"remote", 0, NULL, 'r'},
+		{"file", 1, NULL, 'f'},
+		{"Nodes", 1, NULL, 'N'},
+		{"Switches", 1, NULL, 'S'},
+		{"Ports", 1, NULL, 'P'},
+		{"Linearcap", 1, NULL, 'L'},
+		{"Mcastcap", 1, NULL, 'M'},
+		{"listen", 1, NULL, 'l'},
+		{"Ignoredups", 0, NULL, 'I'},
+		{"start", 0, NULL, 's'},
+		{"debug", 0, NULL, 'd'},
+		{"parsedebug", 0, NULL, 'p'},
+		{"verbose", 0, NULL, 'v'},
+		{"Version", 0, NULL, 'V'},
+		{"no-console", 0, NULL, 'n'},
+		{"help", 0, NULL, 'h'},
+		{"usage", 0, NULL, 'u'},
 		{}
 	};
 
@@ -773,22 +731,25 @@ int main(int argc, char **argv)
 			ignoreduplicate = 1;
 			break;
 		case 'N':
-			maxnetnodes = strtoul(optarg, 0, 0);
+			maxnetnodes = strtoul(optarg, NULL, 0);
 			break;
 		case 'S':
-			maxnetswitches = strtoul(optarg, 0, 0);
+			maxnetswitches = strtoul(optarg, NULL, 0);
 			break;
 		case 'P':
-			maxnetports = strtoul(optarg, 0, 0);
+			maxnetports = strtoul(optarg, NULL, 0);
 			break;
 		case 'L':
-			maxlinearcap = strtoul(optarg, 0, 0);
+			maxlinearcap = strtoul(optarg, NULL, 0);
 			break;
 		case 'M':
-			maxmcastcap = strtoul(optarg, 0, 0);
+			maxmcastcap = strtoul(optarg, NULL, 0);
 			break;
 	        case 'l':
-			listen_to_port = strtoul(optarg, 0, 0);
+			listen_to_port = strtoul(optarg, NULL, 0);
+			break;
+		case 'n':
+			console = 0;
 			break;
 		case 'V':
 		default:
@@ -800,9 +761,12 @@ int main(int argc, char **argv)
 
 	infile = stdin;
 	outfile = stdout;
-	if (outfname && (outfile = fopen(outfname, "w")) == 0)
+	if (outfname && (outfile = fopen(outfname, "w")) == NULL)
 		IBPANIC("can't open out file %s for write", outfname);
 
+	if (!console && !netstarted)
+		IBPANIC("Cannot disable console without starting "
+				"the network from the command line");
 	if (optind >= argc)
 		usage(argv[0]);
 
