@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2004-2008 Voltaire, Inc. All rights reserved.
+ * Copyright (c) 2011 Mellanox Technologies LTD. All rights reserved.
  *
  * This file is part of ibsim.
  *
@@ -42,25 +43,74 @@
 #define MAXNETPORTS	(MAXNETSWITCHS*36+MAXNETNODES*2)
 #define MAXNETALIASES	MAXNETPORTS
 
-#define MAXLINEARCAP	(30*1024)
+#define MAXLINEARCAP	(48*1024)
 #define MAXMCASTCAP	1024
 #define LASTBLOCK32	(MAXMCASTCAP/32-1)
 #define MCASTMASKSIZE    16
-// linkwidth == 4X - must be one width only 1,2 or 8
-#define LINKWIDTH_1x        1
-#define LINKWIDTH_4x        2
-#define LINKWIDTH_8x        4
-#define LINKWIDTH_12x       8
-#define LINKWIDTH_1x_4x     3
-#define LINKWIDTH_1x_4x_12x 11
+// linkwidth == 4X - must be one width only 1,2,8 or 16
+#define LINKWIDTH_1x               1
+#define LINKWIDTH_4x               2
+#define LINKWIDTH_1x_4x            3
+#define LINKWIDTH_8x               4
+#define LINKWIDTH_1x_8x            5
+#define LINKWIDTH_4x_8x            6
+#define LINKWIDTH_1x_4x_8x         7
+#define LINKWIDTH_12x              8
+#define LINKWIDTH_1x_12x           9
+#define LINKWIDTH_4x_12x          10
+#define LINKWIDTH_1x_4x_12x       11
+#define LINKWIDTH_8x_12x          12
+#define LINKWIDTH_1x_8x_12x       13
+#define LINKWIDTH_4x_8x_12x       14
+#define LINKWIDTH_1x_4x_8x_12x    15
+#define LINKWIDTH_2x              16
+#define LINKWIDTH_1x_2x           17
+#define LINKWIDTH_2x_4x           18
+#define LINKWIDTH_1x_2x_4x        19
+#define LINKWIDTH_2x_8x           20
+#define LINKWIDTH_1x_2x_8x        21
+#define LINKWIDTH_2x_4x_8x        22
+#define LINKWIDTH_1x_2x_4x_8x     23
+#define LINKWIDTH_2x_12x          24
+#define LINKWIDTH_1x_2x_12x       25
+#define LINKWIDTH_2x_4x_12x       26
+#define LINKWIDTH_1x_2x_4x_12x    27
+#define LINKWIDTH_2x_8x_12x       28
+#define LINKWIDTH_1x_2x_8x_12x    29
+#define LINKWIDTH_2x_4x_8x_12x    30
+#define LINKWIDTH_1x_2x_4x_8x_12x 31
 
-#define LINKSPEED_SDR       1
-#define LINKSPEED_DDR       2
-#define LINKSPEED_QDR       4
-#define LINKSPEED_SDR_DDR   3
+#define LINKSPEED_SDR         1
+#define LINKSPEED_DDR         2
+#define LINKSPEED_SDR_DDR     3
+#define LINKSPEED_QDR         4
+#define LINKSPEED_SDR_QDR     5
+#define LINKSPEED_SDR_DDR_QDR 7
+
+#define LINKSPEEDEXT_NONE             0
+#define LINKSPEEDEXT_FDR              1
+#define LINKSPEEDEXT_EDR              2
+#define LINKSPEEDEXT_FDR_EDR          3
+#define LINKSPEEDEXT_HDR              4
+#define LINKSPEEDEXT_HDR_FDR          5
+#define LINKSPEEDEXT_HDR_EDR          6
+#define LINKSPEEDEXT_HDR_EDR_FDR      7
+#define LINKSPEEDEXT_NDR              8
+#define LINKSPEEDEXT_NDR_FDR          9
+#define LINKSPEEDEXT_NDR_EDR         10
+#define LINKSPEEDEXT_NDR_FDR_EDR     11
+#define LINKSPEEDEXT_NDR_HDR         12
+#define LINKSPEEDEXT_NDR_HDR_FDR     13
+#define LINKSPEEDEXT_NDR_HDR_EDR     14
+#define LINKSPEEDEXT_NDR_HDR_EDR_FDR 15
+
+#define MLNXLINKSPEED_NONE	0
+#define MLNXLINKSPEED_FDR10	1
 
 #define	DEFAULT_LINKWIDTH	LINKWIDTH_4x
 #define DEFAULT_LINKSPEED	LINKSPEED_SDR
+#define DEFAULT_LINKSPEEDEXT	LINKSPEEDEXT_NONE
+#define DEFAULT_MLNXLINKSPEED MLNXLINKSPEED_NONE
 
 #define NODEPREFIX	20
 #define NODEIDLEN	65
@@ -89,9 +139,17 @@ enum PORTINFO_CAPMASK {
 	CAPMASK_ISSM = (1<<1),
 	CAPMASK_ISNOTICE = (1<<2),
 	CAPMASK_ISTRAP = (1<<3),
+	CAPMASK_ISEXTENDEDSPEEDS = (1<<14),
+	CAPMASK_ISCAPMASK2 = (1<<15),
 	CAPMASK_ISCAPMASKTRAP = (1<<22),
 };
 
+/* some PortInfo capmask2 fields */
+enum PORTINFO_CAPMASK2 {
+	CAPMASK2_ISLINKWIDTH2X = (1<<4),
+	CAPMASK2_ISLINKSPEEDHDR = (1<<5),
+};
+
 enum GS_PERF_COUNTER_SELECT_MASK {
 	GS_PERF_ERR_SYM_MASK = (1UL << 0),	// SYMBOL_ERROR_COUNTER
 	GS_PERF_LINK_RECOVERS_MASK = (1UL << 1),	// LINK_ERROR_RECOVERY_COUNTER
@@ -112,6 +170,18 @@ enum GS_PERF_COUNTER_SELECT_MASK {
 	GS_PERF_XMT_WAIT_MASK = (1UL << 16),	// PORT_XMIT_WAIT
 };
 
+// Infiniband ClassPortInfo capmask masks
+enum IB_CPI_CAPMASK {
+	IB_PM_ALL_PORT_SELECT = (1UL << 8),
+	IB_PM_EXT_WIDTH_SUPPORTED = (1UL << 9),
+	IB_PM_EXT_WIDTH_NOIETF_SUP = (1UL << 10),
+	IB_PM_SAMPLES_ONLY_SUP = (1UL << 11),
+	IB_PM_PC_XMIT_WAIT_SUP = (1UL << 12),
+	IS_PM_INH_LMTD_PKEY_MC_CONSTR_ERR = (1UL << 13),
+	IS_PM_RSFEC_COUNTERS_SUP = (1UL << 14),
+	IB_PM_IS_QP1_DROP_SUP = (1UL << 15),
+};
+
 enum GS_PC_EXT_SELECT_MASK {
 	GS_PC_EXT_XMIT_DATA = 1 << 0,
 	GS_PC_EXT_RECV_DATA = 1 << 1,
@@ -202,6 +272,7 @@ struct Portinfo {
 	int localport;
 	int linkwidthen;
 	int linkspeeden;
+	int linkspeedexten;
 };
 
 struct Portcounters {
@@ -275,6 +346,7 @@ struct Portcounters {
 };
 
 struct Port {
+	uint64_t subnet_prefix;
 	uint64_t portguid;
 	int portnum;
 	int lid;
@@ -283,11 +355,16 @@ struct Port {
 	int linkwidthena;
 	int linkspeed;
 	int linkspeedena;
+	int linkspeedext;
+	int linkspeedextena;
+	int mlnx_linkspeed;
+	int mlnx_linkspeedena;
 	int state;
 	int physstate;
 	int lmc;
 	int hoqlife;
 	uint8_t portinfo[64];
+	uint8_t extportinfo[64];
 	uint8_t op_vls;
 
 	char remotenodeid[NODEIDLEN];
@@ -369,11 +446,11 @@ char *expand_name(char *base, char *name, char **portstr);
 int read_netconf(char *name, FILE * out);
 int set_default_port(char *nodeid);
 int readline(int fd, char *buf, int sz);
+int alloc_core(void);
+void free_core(void);
 
 // sim_cmd.c
 int do_cmd(char *buf, FILE *f);
-int sim_cmd_file(FILE * file, char *line);
-void *sim_cmd_thread(void *file);
 
 // sim_mad.c
 int process_packet(Client * cl, void *p, int size, Client ** dcl);
@@ -382,5 +459,17 @@ int send_trap(Port * port, unsigned trapnum);
 extern Port *default_port;
 extern int simverb;
 extern int netstarted;
-
+extern int maxnetnodes;
+extern int maxnetswitches;
+extern int maxnetports;
+extern int maxlinearcap;
+extern int maxmcastcap;
+extern int maxnetaliases;
+extern int ignoreduplicate;
+extern Node *nodes;
+extern Switch *switches;
+extern Port *ports;
+extern Port **lids;
+extern int netnodes, netports, netswitches;
+extern int parsedebug;
 #endif				/* __SIM_H__ */
