@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2004-2008 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2009 HNR Consulting. All rights reserved.
+ * Copyright (c) 2011 Mellanox Technologies LTD. All rights reserved.
  *
  * This file is part of ibsim.
  *
@@ -42,6 +43,7 @@
 #include <errno.h>
 #include <netinet/in.h>
 #include <inttypes.h>
+#include <limits.h>
 
 #include <ibsim.h>
 #include "sim.h"
@@ -62,49 +64,103 @@ static Smpfn do_nodeinfo, do_nodedesc, do_switchinfo, do_portinfo,
     do_rcv_error_details, do_xmit_discard_details, do_op_rcv_counters,
     do_flow_ctl_counters, do_vl_op_packets, do_vl_op_data,
     do_vl_xmit_flow_ctl_update_errors, do_vl_xmit_wait_counters, do_pkeytbl,
-    do_sl2vl, do_vlarb, do_guidinfo, do_cpi;
+    do_sl2vl, do_vlarb, do_guidinfo, do_cpi, do_extportinfo;
 
 static EncodeTrapfn encode_trap128;
 static EncodeTrapfn encode_trap144;
 
-static Smpfn *attrs[IB_PERFORMANCE_CLASS + 1][0xff] = {
-	[IB_SMI_CLASS] {[IB_ATTR_NODE_DESC] do_nodedesc,
-			[IB_ATTR_NODE_INFO] do_nodeinfo,
-			[IB_ATTR_SWITCH_INFO] do_switchinfo,
-			[IB_ATTR_PORT_INFO] do_portinfo,
-			[IB_ATTR_LINEARFORWTBL] do_linearforwtbl,
-			[IB_ATTR_MULTICASTFORWTBL] do_multicastforwtbl,
-			[IB_ATTR_PKEY_TBL] do_pkeytbl,
-			[IB_ATTR_SLVL_TABLE] do_sl2vl,
-			[IB_ATTR_VL_ARBITRATION] do_vlarb,
-			[IB_ATTR_GUID_INFO] do_guidinfo,
-			[IB_ATTR_SMINFO] NULL,
 
-			[IB_ATTR_LAST] 0,
-			},
-	[IB_PERFORMANCE_CLASS] {[CLASS_PORT_INFO] = do_cpi,
-				[IB_GSI_PORT_SAMPLES_CONTROL] = 0,
-				[IB_GSI_PORT_SAMPLES_RESULT] = 0,
-				[IB_GSI_PORT_COUNTERS] = do_portcounters,
-				[IB_GSI_PORT_COUNTERS_EXT] = do_extcounters,
-				[IB_GSI_PORT_RCV_ERROR_DETAILS] = do_rcv_error_details,
-				[IB_GSI_PORT_XMIT_DISCARD_DETAILS] = do_xmit_discard_details,
-				[IB_GSI_PORT_PORT_OP_RCV_COUNTERS] = do_op_rcv_counters,
-				[IB_GSI_PORT_PORT_FLOW_CTL_COUNTERS] = do_flow_ctl_counters,
-				[IB_GSI_PORT_PORT_VL_OP_PACKETS] = do_vl_op_packets,
-				[IB_GSI_PORT_PORT_VL_OP_DATA] = do_vl_op_data,
-				[IB_GSI_PORT_PORT_VL_XMIT_FLOW_CTL_UPDATE_ERRORS] = do_vl_xmit_flow_ctl_update_errors,
-				[IB_GSI_PORT_PORT_VL_XMIT_WAIT_COUNTERS] = do_vl_xmit_wait_counters,
+#define ATTRIBUTES_NUMBER 20
 
-				[IB_GSI_ATTR_LAST] 0,
-				},
+typedef struct {
+	unsigned attr_id;
+	Smpfn * handler;
+} attr_handler;
+
+typedef struct {
+	unsigned class_id;
+	attr_handler handlers[ATTRIBUTES_NUMBER];
+} class_handler;
+
+static class_handler smp_handlers_array[] = {
+	{IB_SMI_CLASS, {
+		{IB_ATTR_NODE_DESC, do_nodedesc},
+		{IB_ATTR_NODE_INFO, do_nodeinfo},
+		{IB_ATTR_SWITCH_INFO, do_switchinfo},
+		{IB_ATTR_PORT_INFO, do_portinfo},
+		{IB_ATTR_LINEARFORWTBL, do_linearforwtbl},
+		{IB_ATTR_MULTICASTFORWTBL, do_multicastforwtbl},
+		{IB_ATTR_PKEY_TBL, do_pkeytbl},
+		{IB_ATTR_SLVL_TABLE, do_sl2vl},
+		{IB_ATTR_VL_ARBITRATION, do_vlarb},
+		{IB_ATTR_GUID_INFO, do_guidinfo},
+		{IB_ATTR_SMINFO, NULL},
+		{IB_ATTR_MLNX_EXT_PORT_INFO, do_extportinfo},
+		{UINT_MAX, NULL}
+		}
+	},
+	{IB_SMI_DIRECT_CLASS, {
+		{IB_ATTR_NODE_DESC, do_nodedesc},
+		{IB_ATTR_NODE_INFO, do_nodeinfo},
+		{IB_ATTR_SWITCH_INFO, do_switchinfo},
+		{IB_ATTR_PORT_INFO, do_portinfo},
+		{IB_ATTR_LINEARFORWTBL, do_linearforwtbl},
+		{IB_ATTR_MULTICASTFORWTBL, do_multicastforwtbl},
+		{IB_ATTR_PKEY_TBL, do_pkeytbl},
+		{IB_ATTR_SLVL_TABLE, do_sl2vl},
+		{IB_ATTR_VL_ARBITRATION, do_vlarb},
+		{IB_ATTR_GUID_INFO, do_guidinfo},
+		{IB_ATTR_SMINFO, NULL},
+		{IB_ATTR_MLNX_EXT_PORT_INFO, do_extportinfo},
+		{UINT_MAX, NULL}
+		}
+	},
+	{IB_PERFORMANCE_CLASS, {
+		{CLASS_PORT_INFO, do_cpi},
+		{IB_GSI_PORT_SAMPLES_CONTROL, NULL},
+		{IB_GSI_PORT_SAMPLES_RESULT, NULL},
+		{IB_GSI_PORT_COUNTERS, do_portcounters},
+		{IB_GSI_PORT_COUNTERS_EXT, do_extcounters},
+		{IB_GSI_PORT_RCV_ERROR_DETAILS, do_rcv_error_details},
+		{IB_GSI_PORT_XMIT_DISCARD_DETAILS, do_xmit_discard_details},
+		{IB_GSI_PORT_PORT_OP_RCV_COUNTERS, do_op_rcv_counters},
+		{IB_GSI_PORT_PORT_FLOW_CTL_COUNTERS, do_flow_ctl_counters},
+		{IB_GSI_PORT_PORT_VL_OP_PACKETS, do_vl_op_packets},
+		{IB_GSI_PORT_PORT_VL_OP_DATA, do_vl_op_data},
+		{IB_GSI_PORT_PORT_VL_XMIT_FLOW_CTL_UPDATE_ERRORS, do_vl_xmit_flow_ctl_update_errors},
+		{IB_GSI_PORT_PORT_VL_XMIT_WAIT_COUNTERS, do_vl_xmit_wait_counters},
+		{UINT_MAX, NULL}
+		}
+	}
+	,
+	{UINT_MAX, {}}
 };
 
-static EncodeTrapfn *encodetrap[] = {
-	[TRAP_128] encode_trap128,
-	[TRAP_144] encode_trap144,
+static Smpfn * get_smp_handler(unsigned class_id, unsigned attr_id)
+{
+	int i, j;
 
-	[TRAP_NUM_LAST] 0,
+	for (i = 0; smp_handlers_array[i].class_id != UINT_MAX; i++ ) {
+
+		if (smp_handlers_array[i].class_id != class_id)
+			continue;
+
+		for (j = 0; smp_handlers_array[i].handlers[j].attr_id != UINT_MAX; j++) {
+			if (smp_handlers_array[i].handlers[j].attr_id != attr_id)
+				continue;
+
+			return smp_handlers_array[i].handlers[j].handler;
+		}
+
+	}
+	return NULL;
+}
+
+static EncodeTrapfn *encodetrap[] = {
+	[TRAP_128] = encode_trap128,
+	[TRAP_144] = encode_trap144,
+
+	[TRAP_NUM_LAST] = NULL,
 };
 
 extern Node *nodes;
@@ -244,9 +300,10 @@ static int do_cpi(Port * port, unsigned op, uint32_t mod, uint8_t * data)
 	mad_set_field(data, 0, IB_CPI_BASEVER_F, 1);
 	mad_set_field(data, 0, IB_CPI_CLASSVER_F, 1);
 	if (node->type != SWITCH_NODE)
-		mad_set_field(data, 0, IB_CPI_CAPMASK_F, 0x200);
+		mad_set_field(data, 0, IB_CPI_CAPMASK_F, IB_PM_EXT_WIDTH_SUPPORTED|IB_PM_PC_XMIT_WAIT_SUP);
 	else
-		mad_set_field(data, 0, IB_CPI_CAPMASK_F, 0x300);
+		mad_set_field(data, 0, IB_CPI_CAPMASK_F, IB_PM_ALL_PORT_SELECT|IB_PM_EXT_WIDTH_SUPPORTED|IB_PM_PC_XMIT_WAIT_SUP);
+
 	mad_set_field(data, 0, IB_CPI_RESP_TIME_VALUE_F, 0x12);
 	return status;
 }
@@ -347,7 +404,7 @@ static int do_sl2vl(Port * port, unsigned op, uint32_t mod, uint8_t * data)
 		n = (mod >> 8) & 0xff;
 		port = node_get_port(port->node, n);
 		n = mod & 0xff;
-		if (!port && !node_get_port(port->node, n))
+		if (!port || !node_get_port(port->node, n))
 			return ERR_BAD_PARAM;
 	} else
 		n = 0;
@@ -426,9 +483,11 @@ static int
 do_portinfo(Port * port, unsigned op, uint32_t portnum, uint8_t * data)
 {
 	Node *node = port->node;
-	Port *p, *rp;
-	int r, newlid, newstate = 0;
+	Port *p, *rp = NULL;
+	int r, newlid;
+	int speed, espeed, width;
 
+	portnum &= 0x7fffffff;
 	if (portnum > node->numports)
 		return ERR_BAD_PARAM;
 
@@ -447,11 +506,11 @@ do_portinfo(Port * port, unsigned op, uint32_t portnum, uint8_t * data)
 		if (newlid != p->lid) {
 			if (p->lid > 0 && p->lid < maxlinearcap
 			    && lids[p->lid] == p)
-				lids[p->lid] = 0;
+				lids[p->lid] = NULL;
 		}
 		p->lid = newlid;
 		p->smlid = mad_get_field(data, 0, IB_PORT_SMLID_F);
-//              p->linkwidth = mad_get_field(data, 0, IB_PORT_LINK_WIDTH_ENABLED_F); // ignored
+              //p->linkwidth = mad_get_field(data, 0, IB_PORT_LINK_WIDTH_ENABLED_F); // ignored
 		p->lmc = mad_get_field(data, 0, IB_PORT_LMC_F);
 		p->hoqlife = mad_get_field(data, 0, IB_PORT_HOQ_LIFE_F);
 		if ((r = mad_get_field(data, 0, IB_PORT_PHYS_STATE_F)))
@@ -460,7 +519,7 @@ do_portinfo(Port * port, unsigned op, uint32_t portnum, uint8_t * data)
 		if (r > 0 && p->remotenode &&
 		    (rp = node_get_port(p->remotenode, p->remoteport))) {
 			if (r == 1) {	/* DOWN */
-				newstate = p->state = 2;	/* set to INIT */
+				p->state = 2;	/* set to INIT */
 				/*
 				 * If the state is changed to initialize (from down or not)
 				 * we should force remote state to same state.
@@ -474,7 +533,7 @@ do_portinfo(Port * port, unsigned op, uint32_t portnum, uint8_t * data)
 			} else if (r > 2) {
 				if (abs(rp->state - r) <= 1
 				    && abs(p->state - r) == 1)
-					newstate = p->state = r;	/* set to new state */
+					p->state = r;	/* set to new state */
 				else
 					return ERR_BAD_PARAM;
 			}
@@ -495,8 +554,139 @@ do_portinfo(Port * port, unsigned op, uint32_t portnum, uint8_t * data)
 		if (val > mad_get_field(data, 0, IB_PORT_VL_CAP_F))
 			return ERR_BAD_PARAM;
 		p->op_vls = val;
+		p->subnet_prefix = mad_get_field64(data, 0, IB_PORT_GID_PREFIX_F);
+
+		if (!rp && p->remotenode)
+			rp = node_get_port(p->remotenode, p->remoteport);
+		else
+			goto update_port;
+
+		speed = mad_get_field(data, 0, IB_PORT_LINK_SPEED_ENABLED_F);
+		switch (speed) {
+		case LINKSPEED_SDR:
+			p->linkspeed = LINKSPEED_SDR;
+			rp->linkspeed = LINKSPEED_SDR;
+			break;
+		case LINKSPEED_SDR_DDR:
+			p->linkspeed = LINKSPEED_DDR;
+			rp->linkspeed = LINKSPEED_DDR;
+			break;
+		case LINKSPEED_SDR_QDR:
+		case LINKSPEED_SDR_DDR_QDR:
+			p->linkspeed = LINKSPEED_QDR;
+			rp->linkspeed = LINKSPEED_QDR;
+			break;
+		default:
+			speed = 0;
+		}
+
+		if (speed && speed != p->linkspeedena)
+			p->linkspeedena = speed;
+		else
+			speed = 0;
+
+		espeed = mad_get_field(data, 0, IB_PORT_LINK_SPEED_EXT_ENABLED_F);
+		switch (espeed) {
+		case LINKSPEEDEXT_FDR:
+			p->linkspeedext = LINKSPEEDEXT_FDR;
+			rp->linkspeedext = LINKSPEEDEXT_FDR;
+			break;
+		case LINKSPEEDEXT_EDR:
+		case LINKSPEEDEXT_FDR_EDR:
+			p->linkspeedext = LINKSPEEDEXT_EDR;
+			rp->linkspeedext = LINKSPEEDEXT_EDR;
+			break;
+		case LINKSPEEDEXT_HDR:
+		case LINKSPEEDEXT_HDR_FDR:
+		case LINKSPEEDEXT_HDR_EDR:
+		case LINKSPEEDEXT_HDR_EDR_FDR:
+			p->linkspeedext = LINKSPEEDEXT_HDR;
+			rp->linkspeedext = LINKSPEEDEXT_HDR;
+			break;
+
+		case LINKSPEEDEXT_NDR:
+		case LINKSPEEDEXT_NDR_FDR:
+		case LINKSPEEDEXT_NDR_EDR:
+		case LINKSPEEDEXT_NDR_FDR_EDR:
+		case LINKSPEEDEXT_NDR_HDR:
+		case LINKSPEEDEXT_NDR_HDR_FDR:
+		case LINKSPEEDEXT_NDR_HDR_EDR:
+		case LINKSPEEDEXT_NDR_HDR_EDR_FDR:
+			p->linkspeedext = LINKSPEEDEXT_NDR;
+			rp->linkspeedext = LINKSPEEDEXT_NDR;
+			break;
+
+		default:
+			espeed = 0;
+		}
+
+		if (espeed && espeed != p->linkspeedextena)
+			p->linkspeedextena = espeed;
+		else
+			espeed = 0;
+
+		width = mad_get_field(data, 0, IB_PORT_LINK_WIDTH_ENABLED_F);
+		switch (width) {
+		case LINKWIDTH_1x:
+			p->linkwidth = LINKWIDTH_1x;
+			rp->linkwidth = LINKWIDTH_1x;
+			break;
+		case LINKWIDTH_4x:
+		case LINKWIDTH_1x_4x:
+		case LINKWIDTH_2x_4x:
+		case LINKWIDTH_1x_2x_4x:
+			p->linkwidth = LINKWIDTH_4x;
+			rp->linkwidth = LINKWIDTH_4x;
+			break;
+		case LINKWIDTH_8x:
+		case LINKWIDTH_1x_8x:
+		case LINKWIDTH_4x_8x:
+		case LINKWIDTH_1x_4x_8x:
+		case LINKWIDTH_2x_8x:
+		case LINKWIDTH_1x_2x_8x:
+		case LINKWIDTH_2x_4x_8x:
+		case LINKWIDTH_1x_2x_4x_8x:
+			p->linkwidth = LINKWIDTH_8x;
+			rp->linkwidth = LINKWIDTH_8x;
+			break;
+		case LINKWIDTH_12x:
+		case LINKWIDTH_1x_12x:
+		case LINKWIDTH_4x_12x:
+		case LINKWIDTH_1x_4x_12x:
+		case LINKWIDTH_8x_12x:
+		case LINKWIDTH_1x_8x_12x:
+		case LINKWIDTH_4x_8x_12x:
+		case LINKWIDTH_1x_4x_8x_12x:
+		case LINKWIDTH_2x_12x:
+		case LINKWIDTH_1x_2x_12x:
+		case LINKWIDTH_2x_4x_12x:
+		case LINKWIDTH_1x_2x_4x_12x:
+		case LINKWIDTH_2x_8x_12x:
+		case LINKWIDTH_1x_2x_8x_12x:
+		case LINKWIDTH_2x_4x_8x_12x:
+		case LINKWIDTH_1x_2x_4x_8x_12x:
+			p->linkwidth = LINKWIDTH_12x;
+			rp->linkwidth = LINKWIDTH_12x;
+			break;
+		case LINKWIDTH_2x:
+		case LINKWIDTH_1x_2x:
+			p->linkwidth = LINKWIDTH_2x;
+			rp->linkwidth = LINKWIDTH_2x;
+			break;
+		default:
+			width = 0;
+		}
+
+		if (width && width != p->linkwidthena)
+			p->linkwidthena = width;
+		else
+			width = 0;
+
+		if (speed || espeed || width)
+			send_trap(port, TRAP_144);
 	}
 
+update_port:
 	update_portinfo(p);
 	memcpy(data, p->portinfo, IB_SMP_DATA_SIZE);
 	mad_set_field(data, 0, IB_PORT_LOCAL_PORT_F, port->portnum);
@@ -504,6 +694,29 @@ do_portinfo(Port * port, unsigned op, uint32_t portnum, uint8_t * data)
 	return 0;
 }
 
+static int
+do_extportinfo(Port * port, unsigned op, uint32_t portnum, uint8_t * data)
+{
+	Node *node = port->node;
+	Port *p;
+
+	if (portnum > node->numports)
+		return ERR_BAD_PARAM;
+
+	if (portnum == 0 && node->type != SWITCH_NODE)  //according to ibspec 14.2.5.6
+		portnum = port->portnum;
+
+	p = node_get_port(node, portnum);
+	DEBUG("in node %" PRIx64 " port %" PRIx64 ": port %" PRIx64 " (%d(%d))",
+	      node->nodeguid, port->portguid, p->portguid, p->portnum, portnum);
+
+	if (op == IB_MAD_METHOD_SET) {
+		memcpy(p->extportinfo, data, IB_SMP_DATA_SIZE);
+	} else
+		memcpy(data, p->extportinfo, IB_SMP_DATA_SIZE);
+
+	return 0;
+}
 static int do_linearforwtbl(Port * port, unsigned op, uint32_t mod,
 			    uint8_t * data)
 {
@@ -512,7 +725,7 @@ static int do_linearforwtbl(Port * port, unsigned op, uint32_t mod,
 	if (!sw)		// not a Switch?
 		return ERR_ATTR_UNSUPPORTED;
 
-	if (mod < 0 || mod > 767)
+	if (mod > 767)
 		return ERR_BAD_PARAM;
 
 	if (op == IB_MAD_METHOD_SET)
@@ -599,7 +812,7 @@ static inline uint32_t addval(uint32_t val, uint32_t delta, uint32_t max)
 #define ADDVAL64(val, add) { uint64_t new = val + add; \
 		val = new < val ? 0xffffffffffffffffULL : new ; }
 
-void pc_add_error_xmitdiscards(Port * port)
+static void pc_add_error_xmitdiscards(Port * port)
 {
 	Portcounters *pc = &(port->portcounters);
 
@@ -607,7 +820,7 @@ void pc_add_error_xmitdiscards(Port * port)
 	    addval(pc->xmitdiscards, 1, GS_PERF_XMT_DISCARDS_LIMIT);
 }
 
-void pc_add_error_rcvswitchrelay(Port * port)
+static void pc_add_error_rcvswitchrelay(Port * port)
 {
 	Portcounters *pc = &(port->portcounters);
 
@@ -615,7 +828,7 @@ void pc_add_error_rcvswitchrelay(Port * port)
 	    addval(pc->errs_rcvswitchrelay, 1, GS_PERF_ERR_SWITCH_REL_LIMIT);
 }
 
-void pc_add_error_errs_rcv(Port * port)
+static void pc_add_error_errs_rcv(Port * port)
 {
 	Portcounters *pc = &(port->portcounters);
 
@@ -1380,7 +1593,7 @@ static int switch_lookup(Node * node, int lid)
 	if (!node->sw)
 		return -1;
 
-	if (lid > node->sw->linearFDBtop || (outport = node->sw->fdb[lid]) < 0) {
+	if (lid > node->sw->linearFDBtop || (outport = node->sw->fdb[lid]) == 0xff) {
 		IBWARN("sw guid %" PRIx64 ": bad lid %u", node->nodeguid, lid);
 		return -1;
 	}
@@ -1440,7 +1653,7 @@ static Port *lid_route_MAD(Port * port, int lid)
 
 	if (lid == 0) {
 		IBWARN("invalid lid 0");
-		return 0;
+		return NULL;
 	}
 
 	if (is_port_lid(port, lid))
@@ -1450,11 +1663,11 @@ static Port *lid_route_MAD(Port * port, int lid)
 		pc_add_error_xmitdiscards(port);
 		IBWARN("failed: disconnected node 0x%" PRIx64 " or port 0x%"
 		       PRIx64 "?", node->nodeguid, port->portguid);
-		return 0;
+		return NULL;
 	}
 
 	if (!pc_updated(&tport, port))	// if Client connected via HCA ...
-		return 0;
+		return NULL;
 
 	for (hop = 0; !is_port_lid(port, lid) && hop < MAXHOPS; hop++) {
 		portnum = switch_lookup(node, lid);
@@ -1463,7 +1676,7 @@ static Port *lid_route_MAD(Port * port, int lid)
 			pc_add_error_rcvswitchrelay(port);
 			DEBUG("illegal lid %u (outport %d node %s ports %d)",
 			      lid, portnum, node->nodeid, node->numports);
-			return 0;
+			return NULL;
 		}
 
 		DEBUG("node %" PRIx64 " outport %d", node->nodeguid, portnum);
@@ -1477,7 +1690,7 @@ static Port *lid_route_MAD(Port * port, int lid)
 
 		if (!link_valid(port)) {
 			pc_add_error_xmitdiscards(port);
-			return 0;
+			return NULL;
 		}
 
 		tport = port;	// prepare to pass PKT to next port
@@ -1488,7 +1701,7 @@ static Port *lid_route_MAD(Port * port, int lid)
 		if (port_get_remote(port, &node, &port) < 0) {
 			pc_add_error_xmitdiscards(tport);
 			IBWARN("no remote");
-			return 0;
+			return NULL;
 		}
 
 		if (!node || !port)	// double check ?...
@@ -1496,11 +1709,11 @@ static Port *lid_route_MAD(Port * port, int lid)
 
 		if (!link_valid(port)) {
 			pc_add_error_xmitdiscards(tport);
-			return 0;
+			return NULL;
 		}
 
 		if (!pc_updated(&tport, port))	//try to transmit PKT
-			return 0;
+			return NULL;
 	}
 
 	DEBUG("routed to node %s port 0x%" PRIx64 " portnum %d (%p)",
@@ -1619,23 +1832,10 @@ static Smpfn *get_handle_fn(ib_rpc_t rpc, int response)
 	Smpfn *fn;
 
 	if (response)
-		return 0;
+		return NULL;
 
-	if (rpc.mgtclass == IB_SMI_CLASS || rpc.mgtclass == IB_SMI_DIRECT_CLASS) {
-		if (rpc.attr.id >= IB_ATTR_LAST
-		    || !(fn = attrs[rpc.mgtclass & 0xf][rpc.attr.id]))
-			return 0;	// attribute/method not supported ???
-		return fn;
-	}
-
-	if (rpc.mgtclass == IB_PERFORMANCE_CLASS) {
-		if (rpc.attr.id >= IB_GSI_ATTR_LAST
-		    || !(fn = attrs[rpc.mgtclass & 0xf][rpc.attr.id]))
-			return 0;	// attribute/method not supported ???
-		return fn;
-	}
-
-	return 0;		// No MGTCLASS matched.
+	fn = get_smp_handler(rpc.mgtclass & 0xf , rpc.attr.id);
+	return fn;
 }
 
 int process_packet(Client * cl, void *p, int size, Client ** dcl)
@@ -1666,7 +1866,7 @@ int process_packet(Client * cl, void *p, int size, Client ** dcl)
 
 	if (rpc.method == 0x7) {
 		IBWARN("lid %u got trap repress - dropping", ntohs(r->dlid));
-		*dcl = 0;
+		*dcl = NULL;
 		return 0;
 	}
 
@@ -1797,7 +1997,7 @@ int send_trap(Port * port, unsigned trapnum)
 	}
 
 	encode_trapfn = encodetrap[trapnum];
-	memset(req.mad, 0, sizeof(req.mad));
+	memset(&req, 0, sizeof(req));
 	encode_trap_header(req.mad);
 	if (encode_trapfn(port, data) < 0)
 		return -1;
@@ -1809,9 +2009,6 @@ int send_trap(Port * port, unsigned trapnum)
 
 	req.dlid = htons(port->smlid);
 	req.slid = htons(lid);
-	req.sqp = 0;
-	req.dqp = 0;
-	req.status = 0;
 	req.length = htonll(sizeof(req.mad));
 
 	// find SM client
