@@ -23,7 +23,7 @@
 #ifdef NOISY_DEBUG
 #define dbg(fmt, arg...) fprintf(stderr, "DBG: " fmt, ##arg )
 #else
-#define dbg(fmt, arg...)
+#define dbg(fmt, arg...) (void)0
 #endif
 
 #define TMO 100
@@ -172,7 +172,7 @@ static uint64_t get_guid_ho(ibmad_gid_t gid)
 {
 	uint64_t guid;
 	memcpy(&guid, &gid[8], sizeof(guid));
-	return ntohll(guid);
+	return be64toh(guid);
 }
 
 static int send_req(struct addr_data *a, uint8_t * umad, int len,
@@ -299,7 +299,7 @@ static int rereg_recv_all(struct addr_data *a, void *umad, int len,
 	uint8_t *mad;
 	uint64_t trid;
 	unsigned n, method, status;
-	int i;
+	unsigned i;
 
 	info("%s...\n", __func__);
 
@@ -346,7 +346,8 @@ static int rereg_query_all(struct addr_data *a, void *umad, int len,
 {
 	uint8_t *mad;
 	unsigned method, status;
-	int i, ret;
+	unsigned i;
+	int ret;
 
 	info("%s...\n", __func__);
 
@@ -511,8 +512,6 @@ static int run_test(const struct test *t, struct test_data *td,
 	ret = t->func(&addr, td);
 
 	umad_unregister(addr.port, addr.agent);
-	umad_close_port(addr.port);
-	umad_done();
 
 	info("\'%s\' %s.\n", t->name, ret ? "failed" : "is done");
 
@@ -521,8 +520,8 @@ static int run_test(const struct test *t, struct test_data *td,
 
 static void make_gid(ibmad_gid_t gid, uint64_t prefix, uint64_t guid)
 {
-	prefix = ntohll(prefix);
-	guid = ntohll(guid);
+	prefix = be64toh(prefix);
+	guid = be64toh(guid);
 	memcpy(&gid[0], &prefix, 8);
 	memcpy(&gid[8], &guid, 8);
 }
@@ -541,9 +540,9 @@ static int make_gids_list(ibmad_gid_t gid, unsigned n, struct gid_list **gid_lis
 	}
 
 	memcpy(&prefix, &gid[0], 8);
-	prefix = ntohll(prefix);
+	prefix = be64toh(prefix);
 	memcpy(&guid, &gid[8], 8);
-	guid = ntohll(guid);
+	guid = be64toh(guid);
 
 	for (i = 0; i <= n; i++)
 		make_gid(list[i].gid, prefix, guid++);
@@ -603,7 +602,7 @@ static int parse_gids_file(const char *guid_file, struct gid_list **gid_list)
 	FILE *f;
 	struct gid_list *list = NULL;
 	unsigned list_size = 0;
-	int i = 0;
+	unsigned i = 0;
 
 	f = fopen(guid_file, "r");
 	if (!f) {
@@ -639,7 +638,8 @@ static int parse_gids_file(const char *guid_file, struct gid_list **gid_list)
 
 static void make_str_opts(char *p, unsigned size, const struct option *o)
 {
-	int i, n = 0;
+	unsigned n;
+	int i;
 
 	for (n = 0; o->name  && n + 2 + o->has_arg < size; o++) {
 		p[n++] = o->val;
@@ -679,32 +679,32 @@ static void usage(char *prog, const struct option *o, const struct test *t)
 int main(int argc, char **argv)
 {
 	const struct option long_opts [] = {
-		{"guidfile", 1, 0, 'g'},
-		{"mgidfile", 1, 0, 'm'},
-		{"GUID", 1, 0, 'G'},
-		{"MGID", 1, 0, 'M'},
-		{"ipv4", 0, 0, 'i'},
-		{"increment", 1, 0, 'I'},
-		{"qkey", 1, 0, 'q'},
-		{"mlid", 1, 0, 'z'},
-		{"mtu", 1, 0, 'y'},
-		{"tclass", 1, 0, 't'},
-		{"pkey", 1, 0, 'p'},
-		{"rate", 1, 0, 'r'},
-		{"sl", 1, 0, 's'},
-		{"flowlabel", 1, 0, 'f'},
-		{"joinstate", 1, 0, 'j'},
-		{"proxy", 0, 0, 'x'},
-		{"version", 0, 0, 'V'},
-		{"verbose", 0, 0, 'v'},
-		{"help", 0, 0, 'h'},
+		{"guidfile", 1, NULL, 'g'},
+		{"mgidfile", 1, NULL, 'm'},
+		{"GUID", 1, NULL, 'G'},
+		{"MGID", 1, NULL, 'M'},
+		{"ipv4", 0, NULL, 'i'},
+		{"increment", 1, NULL, 'I'},
+		{"qkey", 1, NULL, 'q'},
+		{"mlid", 1, NULL, 'z'},
+		{"mtu", 1, NULL, 'y'},
+		{"tclass", 1, NULL, 't'},
+		{"pkey", 1, NULL, 'p'},
+		{"rate", 1, NULL, 'r'},
+		{"sl", 1, NULL, 's'},
+		{"flowlabel", 1, NULL, 'f'},
+		{"joinstate", 1, NULL, 'j'},
+		{"proxy", 0, NULL, 'x'},
+		{"version", 0, NULL, 'V'},
+		{"verbose", 0, NULL, 'v'},
+		{"help", 0, NULL, 'h'},
 		{}
 	};
 	const struct test tests[] = {
 		{"rereg", run_port_rereg_test, "simulates port reregistration"},
 		{"joins", run_mcast_joins_test, "run a lot of join requests"},
 		{"leave", run_mcast_leave_test, "run a lot of leave requests"},
-		{0}
+		{NULL, NULL, NULL}
 	};
 
 	char opt_str[256];
